import subprocess
from pathlib import Path
from typing import runtime_checkable, Protocol, List


@runtime_checkable
class ElementProtocol(Protocol):
    def check_dependencies(self) -> None: ...
    def bootstrap(self, element_path: Path, url: str, username: str, password: str, clean: bool, global_python_env: Path, single_element: bool) -> None: ...
    def get_build_dependencies(self) -> List[str]: ...
    def build(self, element_path: Path) -> None: ...
    def deploy(self, element_path: Path, url: str, username: str, password: str) -> None: ...
    def get_files_to_package(self, element_path: Path) -> List[str]: ...
    def watch(self, element_path: Path, url: str, username: str, password: str) -> subprocess.Popen: ...
    def test(self, element_path: Path) -> None: ...
